"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useLinks = void 0;
const objects_1 = require("@hypernetlabs/objects");
const contexts_1 = require("../contexts");
const ethers_1 = require("ethers");
const react_1 = require("react");
const react_alert_1 = require("react-alert");
var EActionTypes;
(function (EActionTypes) {
    EActionTypes["FETCHING"] = "FETCHING";
    EActionTypes["FETCHED"] = "FETCHED";
    EActionTypes["PUBLIC_IDENTIFIER_FETCHED"] = "PUBLIC_IDENTIFIER_FETCHED";
    EActionTypes["ERROR"] = "ERROR";
})(EActionTypes || (EActionTypes = {}));
function useLinks() {
    const { coreProxy } = contexts_1.useStoreContext();
    const alert = react_alert_1.useAlert();
    const initialState = {
        loading: true,
        error: null,
        links: [],
        publicIdentifier: objects_1.PublicIdentifier(""),
        acceptPayment,
        disputePayment,
        pullFunds,
    };
    const [state, dispatch] = react_1.useReducer((state, action) => {
        switch (action.type) {
            case EActionTypes.FETCHING:
                return Object.assign(Object.assign({}, state), { loading: true });
            case EActionTypes.FETCHED:
                return Object.assign(Object.assign({}, state), { loading: false, links: action.payload });
            case EActionTypes.PUBLIC_IDENTIFIER_FETCHED:
                return Object.assign(Object.assign({}, state), { loading: false, publicIdentifier: action.payload });
            case EActionTypes.ERROR:
                return Object.assign(Object.assign({}, state), { loading: false, error: action.payload });
            default:
                return state;
        }
    }, initialState);
    react_1.useEffect(() => {
        fetchData();
        coreProxy.onPullPaymentSent.subscribe({
            next: onPullPaymentSent,
        });
        coreProxy.onPushPaymentSent.subscribe({
            next: onPushPaymentSent,
        });
    }, []);
    function fetchData() {
        dispatch({ type: EActionTypes.FETCHING });
        coreProxy.getLinks().match((links) => {
            dispatch({ type: EActionTypes.FETCHED, payload: [...links] });
        }, handleError);
        coreProxy.getPublicIdentifier().match((publicIdentifier) => {
            dispatch({
                type: EActionTypes.PUBLIC_IDENTIFIER_FETCHED,
                payload: publicIdentifier,
            });
        }, handleError);
    }
    function onPullPaymentSent(payment) {
        const linksArr = [...state.links];
        // Check if there is already a link for the payment counterparty
        const paymentLinkIndex = linksArr.findIndex((val) => {
            const counterPartyAccount = val.counterPartyAccount;
            return (counterPartyAccount === payment.to ||
                counterPartyAccount === payment.from);
        });
        if (paymentLinkIndex === -1) {
            // We need to create a new link for the counterparty
            const counterPartyAccount = payment.to === state.publicIdentifier ? payment.from : payment.to;
            linksArr.push(new objects_1.HypernetLink(counterPartyAccount, [payment], [], [payment], [], [payment]));
        }
        else {
            // It's for us, we'll need to add it to the payments for the link
            linksArr[paymentLinkIndex].pullPayments.push(payment);
            linksArr[paymentLinkIndex].payments.push(payment);
        }
        dispatch({ type: EActionTypes.FETCHED, payload: [...linksArr] });
    }
    function onPushPaymentSent(payment) {
        const linksArr = [...state.links];
        // Check if there is already a link for the payment counterparty
        const paymentLinkIndex = linksArr.findIndex((val) => {
            const counterPartyAccount = val.counterPartyAccount;
            return (counterPartyAccount === payment.to ||
                counterPartyAccount === payment.from);
        });
        if (paymentLinkIndex === -1) {
            // We need to create a new link for the counterparty
            const counterPartyAccount = payment.to === state.publicIdentifier ? payment.from : payment.to;
            linksArr.push(new objects_1.HypernetLink(counterPartyAccount, [payment], [payment], [], [payment], []));
        }
        else {
            // It's for us, we'll need to add it to the payments for the link
            linksArr[paymentLinkIndex].pushPayments.push(payment);
            linksArr[paymentLinkIndex].payments.push(payment);
        }
        dispatch({ type: EActionTypes.FETCHED, payload: [...linksArr] });
    }
    function acceptPayment(paymentId) {
        coreProxy.acceptOffers([paymentId]).match((results) => {
            try {
                results[0].match(() => {
                    fetchData();
                    alert.success("Payment accepted successfully!");
                }, handleError);
            }
            catch (err) {
                handleError();
            }
        }, handleError);
    }
    function disputePayment(paymentId) {
        coreProxy.initiateDispute(paymentId).match(() => {
            fetchData();
            alert.success("Payment disputed successfully!");
        }, handleError);
    }
    function pullFunds(paymentId) {
        coreProxy.pullFunds(paymentId, ethers_1.BigNumber.from(1)).match(() => {
            fetchData();
            alert.success("Payment disputed successfully!");
        }, handleError);
    }
    function handleError(error) {
        const err = (error === null || error === void 0 ? void 0 : error.message) || "Something went wrong in payments!";
        alert.error(err);
        dispatch({ type: EActionTypes.ERROR, payload: err });
    }
    return state;
}
exports.useLinks = useLinks;
//# sourceMappingURL=useLinks.js.map